/*
 * Papillon - Solaris security module -  http://www.roqe.org/papillon
 * Copyright (c) 2000-2002 Konrad Rieck <kr@roqe.org>
 * $Id: refcnt.c,v 1.3 2002/09/02 09:33:41 kr Exp $
 *
 * An implementation of a reference counter, in order to keep track 
 * who is calling the syscalls on smp machines. Thanks to Acpizer for
 * his code. 
 */

#define _KERNEL

#include "headers.h"

extern kmutex_t refcnt_lock;
static int refcount;

void inc_refcnt()
{
    ASSERT(refcount >= 0);
    mutex_enter(&refcnt_lock);
    refcount++;
    mutex_exit(&refcnt_lock);
}

void dec_refcnt()
{
    mutex_enter(&refcnt_lock);
    refcount--;
    mutex_exit(&refcnt_lock);
    ASSERT(refcount >= 0);
}

void init_refcnt()
{
    mutex_enter(&refcnt_lock);
    refcount = 0;
    mutex_exit(&refcnt_lock);
}

boolean_t check_refcnt()
{
    mutex_enter(&refcnt_lock);
    if (refcount > 0) {
	mutex_exit(&refcnt_lock);
	return B_FALSE;
    }

    mutex_exit(&refcnt_lock);
    return B_TRUE;
}
