/*
 * Papillon - Solaris security module -  http://www.roqe.org/papillon
 * Copyright (c) 2000-2002 Konrad Rieck <kr@roqe.org>
 * $Id: rproc.c,v 1.13 2002/09/02 09:33:41 kr Exp $
 *
 * This file contains the restricted proc routines. It is implemented by
 * redirecting calls to some proc vnode operations and checking permissions.
 */

#define _KERNEL

#include "headers.h"

#if defined(RSTPROC)

extern pap_config_t config;
extern int (*old_praccess) ();
extern krwlock_t config_rwlock;

int pap_praccess(struct vnode *vp, int mode, int flags, struct cred *cr)
{
    boolean_t access = B_TRUE;
    int error;
    prnode_t *pr;

    inc_refcnt();
    dcmn_err((CE_CONT, "praccess() vnode operation.\n"));
    rw_enter(&config_rwlock, RW_READER);

    if (!config.rstproc)
	goto skip;

    pr = VTOP(vp);

    mutex_enter(&pr->pr_mutex);

    if (pr && pr->pr_common && pr->pr_common->prc_proc &&
	pr->pr_type != PR_PROCDIR &&
	cr->cr_ruid != SUSER_UID &&
	cr->cr_rgid != SUSER_GID &&
	cr->cr_ruid != pr->pr_common->prc_proc->p_cred->cr_ruid)
	access = B_FALSE;

    mutex_exit(&pr->pr_mutex);


  skip:
    if (access)
	error = old_praccess(vp, mode, flags, cr);
    else
	error = set_errno(EPERM);

    rw_exit(&config_rwlock);
    dec_refcnt();

    return error;
}

#endif				/* RSTPROC */
