/*
 * Papillon - Solaris security module -  http://www.roqe.org/papillon
 * Copyright (c) 2000-2002 Konrad Rieck <kr@roqe.org>
 * $Id: suid.c,v 1.14 2002/09/02 09:33:41 kr Exp $
 * 
 * This file contains the STDIO protection that only applies to suid/sgid
 * files.
 */

#define _KERNEL

#include "headers.h"

#if defined(SECSTDFD)

#define STDIO(i) i==0?"STDIN":i==1?"STDOUT":"STDERR"

extern pap_syscall_t syscalls[];

extern pap_config_t config;
extern krwlock_t config_rwlock;

int suid_check_vnode(vnode_t * vp)
{
    vattr_t vattr;
    boolean_t ret = B_FALSE;

    if (VOP_GETATTR(vp, &vattr, 0, CRED()))
	goto err;

    if (vattr.va_mode & (S_ISUID | S_ISGID))
	ret = B_TRUE;

  err:
    return ret;
}

int suid_check_path(char *path)
{
    vnode_t *vp;
    vattr_t vattr;
    boolean_t ret = B_FALSE;

    if (lookupname((char *) path, UIO_USERSPACE, NO_FOLLOW, NULL, &vp))
	return B_FALSE;

    ret = suid_check_vnode(vp);

    VN_RELE(vp);
    return ret;
}

int64_t pap_execve(const char *path, char *const argv[], char *const envp[])
{
    uf_info_t *ufinfo;
    short i;
    boolean_t suid = B_FALSE, fake_opened_fd[3];
    char k_path[256];
    int64_t ret;

    inc_refcnt();
    dcmn_err((CE_CONT, "execve() syscall.\n"));
    rw_enter(&config_rwlock, RW_READER);

    if (!config.secstdfd)
	goto skip_secstdfd;

    suid = suid_check_path((char *) path);
    ufinfo = P_FINFO(curproc);

    for (i = 0; suid && ufinfo && i < 3; i++) {
	if (!ufinfo->fi_list[i].uf_file) {
	    if (!copyinstr(path, k_path, 255, (size_t *) & ret))
		cmn_err(CE_WARN,
			"%s closed while executing suid/sgid binary %s from "
			"%s (pid %d, uid %d), fake opening %s.", STDIO(i),
			k_path, u.u_psargs, ddi_get_pid(), CRED()->cr_ruid,
			STDIO(i));
	    ufalloc(i);
	    fake_opened_fd[i] = B_TRUE;
	} else
	    fake_opened_fd[i] = B_FALSE;
    }

  skip_secstdfd:
    ret = syscalls[EXECVE].sc(path, argv, envp);

    for (i = 0; config.secstdfd && suid && ufinfo && i < 3; i++)
	if (fake_opened_fd[i]) {
	    cmn_err(CE_WARN, "Closing fake opened %s.", STDIO(i));
	    closeandsetf(i, NULL);
	}

    rw_exit(&config_rwlock);
    dec_refcnt();

    return ret;
}

#endif				/* SECSTDFD */
